@@ -64,8 +64,6 @@ static void terminal_preferences_dialog_background_set    (GtkFileChooserButton
                                                            TerminalPreferencesDialog *dialog);
 static void terminal_preferences_dialog_encoding_changed  (GtkComboBox               *combobox,
                                                            TerminalPreferencesDialog *dialog);
-static void terminal_preferences_dialog_scroll_unlimited  (GtkWidget                 *button,
-                                                           TerminalPreferencesDialog *dialog);
 
 
 
@@ -138,24 +136,27 @@ terminal_preferences_dialog_init (TerminalPreferencesDialog *dialog)
   gchar            *current;
   GtkTreeIter       current_iter;
   const gchar      *props_active[] = { "title-mode", "command-login-shell",
+                                       "run-custom-command",
                                        "scrolling-on-output", "scrolling-on-keystroke",
                                        "scrolling-bar", "scrolling-unlimited",
                                        "misc-cursor-shape", "misc-cursor-blinks",
-                                       "font-allow-bold", "misc-menubar-default",
-                                       "misc-toolbar-default", "misc-borders-default",
-                                       "misc-tab-close-middle-click", "misc-mouse-autohide",
-                                       "misc-rewrap-on-resize", "shortcuts-no-helpkey",
+                                       "font-allow-bold", "font-use-system",
+                                       "misc-menubar-default", "misc-toolbar-default",
+                                       "misc-borders-default", "misc-tab-close-middle-click",
+                                       "misc-mouse-autohide", "misc-rewrap-on-resize",
+                                       "misc-copy-on-select", "shortcuts-no-helpkey",
                                        "shortcuts-no-mnemonics", "shortcuts-no-menukey",
                                        "binding-backspace", "binding-delete",
-                                       "background-mode", "background-image-style",
-                                       "color-background-vary", "dropdown-keep-open-default",
-                                       "dropdown-keep-above", "dropdown-toggle-focus",
-                                       "dropdown-status-icon", "dropdown-move-to-active",
-                                       "dropdown-always-show-tabs"
+                                       "binding-ambiguous-width", "background-mode",
+                                       "background-image-style", "color-background-vary",
+                                       "dropdown-keep-open-default", "dropdown-keep-above",
+                                       "dropdown-toggle-focus", "dropdown-status-icon",
+                                       "dropdown-move-to-active", "dropdown-always-show-tabs"
                                      };
   const gchar      *props_color[] =  { "color-foreground", "color-cursor",
                                        "color-background", "tab-activity-color",
-                                       "color-selection", "color-bold"
+                                       "color-selection", "color-selection-bg",
+                                       "color-bold"
                                      };
   const gchar      *props_value[] =  { "dropdown-height", "dropdown-width",
                                        "dropdown-position", "dropdown-opacity",
@@ -231,19 +232,36 @@ error:
   /* other properties */
   BIND_PROPERTIES ("font-name", "font-name");
   BIND_PROPERTIES ("title-initial", "text");
+  BIND_PROPERTIES ("custom-command", "text");
   BIND_PROPERTIES ("word-chars", "text");
   BIND_PROPERTIES ("scrolling-lines", "value");
   BIND_PROPERTIES ("tab-activity-timeout", "value");
   BIND_PROPERTIES ("background-darkness", "value");
   BIND_PROPERTIES ("background-image-shading", "value");
 
+  /* run custom command button */
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "run-custom-command");
+  object2 = gtk_builder_get_object (GTK_BUILDER (dialog), "hbox3");
+  terminal_return_if_fail (G_IS_OBJECT (object) && G_IS_OBJECT (object2));
+  g_object_bind_property (G_OBJECT (object), "active",
+                          G_OBJECT (object2), "sensitive",
+                          G_BINDING_SYNC_CREATE);
+
   /* unlimited scrollback button */
   object = gtk_builder_get_object (GTK_BUILDER (dialog), "scrolling-unlimited");
-  terminal_return_if_fail (G_IS_OBJECT (object));
-  g_signal_connect (G_OBJECT (object), "realize",
-      G_CALLBACK (terminal_preferences_dialog_scroll_unlimited), dialog);
-  g_signal_connect (G_OBJECT (object), "clicked",
-      G_CALLBACK (terminal_preferences_dialog_scroll_unlimited), dialog);
+  object2 = gtk_builder_get_object (GTK_BUILDER (dialog), "scrolling-lines");
+  terminal_return_if_fail (G_IS_OBJECT (object) && G_IS_OBJECT (object2));
+  g_object_bind_property (G_OBJECT (object), "active",
+                          G_OBJECT (object2), "sensitive",
+                          G_BINDING_INVERT_BOOLEAN | G_BINDING_SYNC_CREATE);
+
+  /* use system font button */
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "font-use-system");
+  object2 = gtk_builder_get_object (GTK_BUILDER (dialog), "font-name");
+  terminal_return_if_fail (G_IS_OBJECT (object) && G_IS_OBJECT (object2));
+  g_object_bind_property (G_OBJECT (object), "active",
+                          G_OBJECT (object2), "sensitive",
+                          G_BINDING_INVERT_BOOLEAN | G_BINDING_SYNC_CREATE);
 
   /* reset comparibility button */
   object = gtk_builder_get_object (GTK_BUILDER (dialog), "reset-compatibility");
@@ -270,6 +288,12 @@ error:
                           G_OBJECT (object), "active",
                           G_BINDING_INVERT_BOOLEAN | G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
   object2 = gtk_builder_get_object (GTK_BUILDER (dialog), "color-selection");
+  terminal_return_if_fail (G_IS_OBJECT (object2));
+  g_object_bind_property (G_OBJECT (object), "active",
+                          G_OBJECT (object2), "sensitive",
+                          G_BINDING_SYNC_CREATE);
+  object2 = gtk_builder_get_object (GTK_BUILDER (dialog), "color-selection-bg");
+  terminal_return_if_fail (G_IS_OBJECT (objec2));
   g_object_bind_property (G_OBJECT (object), "active",
                           G_OBJECT (object2), "sensitive",
                           G_BINDING_SYNC_CREATE);
@@ -280,6 +304,7 @@ error:
                           G_OBJECT (object), "active",
                           G_BINDING_INVERT_BOOLEAN | G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
   object2 = gtk_builder_get_object (GTK_BUILDER (dialog), "color-bold");
+  terminal_return_if_fail (G_IS_OBJECT (object2));
   g_object_bind_property (G_OBJECT (object), "active",
                           G_OBJECT (object2), "sensitive",
                           G_BINDING_SYNC_CREATE);
@@ -794,7 +819,7 @@ terminal_preferences_dialog_reset_compat (GtkWidget                 *button,
 {
   GParamSpec  *spec;
   GValue       value = { 0, };
-  const gchar *properties[] = { "binding-backspace", "binding-delete" };
+  const gchar *properties[] = { "binding-backspace", "binding-delete", "binding-ambiguous-width" };
   guint        i;
 
   for (i = 0; i < G_N_ELEMENTS (properties); i++)
@@ -844,7 +869,7 @@ terminal_preferences_dialog_background_mode (GtkWidget                 *combobox
   terminal_return_if_fail (GTK_IS_COMBO_BOX (combobox));
 
   active = gtk_combo_box_get_active (GTK_COMBO_BOX (combobox));
-  composited = gtk_widget_is_composited (combobox);
+  composited = gdk_screen_is_composited (gtk_widget_get_screen (combobox));
 
   object = gtk_builder_get_object (GTK_BUILDER (dialog), "box-file");
   terminal_return_if_fail (G_IS_OBJECT (object));
@@ -936,25 +961,6 @@ terminal_preferences_dialog_encoding_changed (GtkComboBox               *combobo
 
 
 
-static void
-terminal_preferences_dialog_scroll_unlimited (GtkWidget                 *button,
-                                              TerminalPreferencesDialog *dialog)
-{
-  GObject *object;
-  gboolean unlimited;
-
-  terminal_return_if_fail (TERMINAL_IS_PREFERENCES_DIALOG (dialog));
-  terminal_return_if_fail (GTK_IS_TOGGLE_BUTTON (widget));
-
-  unlimited = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button));
-
-  object = gtk_builder_get_object (GTK_BUILDER (dialog), "scrolling-lines");
-  terminal_return_if_fail (G_IS_OBJECT (object));
-  g_object_set (G_OBJECT (object), "sensitive", !unlimited, NULL);
-}
-
-
-
 /**
  * terminal_preferences_dialog_new:
  *
@@ -967,7 +973,6 @@ terminal_preferences_dialog_new (gboolean show_drop_down)
   GObject    *dialog;
   GObject    *object;
   GObject    *notebook;
-  gboolean    composited;
 
   builder = g_object_new (TERMINAL_TYPE_PREFERENCES_DIALOG, NULL);
 
@@ -984,8 +989,7 @@ terminal_preferences_dialog_new (gboolean show_drop_down)
           gtk_notebook_page_num (GTK_NOTEBOOK (notebook), GTK_WIDGET (object)));
 
       /* show warning and disable control if WM does not support compositing */
-      composited = gtk_widget_is_composited (GTK_WIDGET (object));
-      if (!composited)
+      if (!gdk_screen_is_composited (gtk_widget_get_screen (GTK_WIDGET (object))))
         {
           object = gtk_builder_get_object (builder, "dropdown-opacity-not-available");
           terminal_return_if_fail (G_IS_OBJECT (object));
