@@ -239,18 +239,18 @@ terminal_window_dropdown_init (TerminalWindowDropdown *dropdown)
   gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dropdown), TRUE);
 
   /* adjust notebook for drop-down usage */
-  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (window->notebook), GTK_POS_BOTTOM);
-  g_object_get (G_OBJECT (window->preferences), "misc-borders-default", &show_borders, NULL);
-  gtk_notebook_set_show_border (GTK_NOTEBOOK (window->notebook), show_borders);
+  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (terminal_window_get_notebook (window)), GTK_POS_BOTTOM);
+  g_object_get (terminal_window_get_preferences (window), "misc-borders-default", &show_borders, NULL);
+  gtk_notebook_set_show_border (GTK_NOTEBOOK (terminal_window_get_notebook (window)), show_borders);
   terminal_window_notebook_show_tabs (window);
 
   /* actions we don't want */
-  action = gtk_action_group_get_action (window->action_group, "show-borders");
+  action = terminal_window_get_action (window, "show-borders");
   gtk_action_set_visible (action, FALSE);
 
   /* notebook buttons */
   hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
-  gtk_notebook_set_action_widget (GTK_NOTEBOOK (window->notebook), hbox, GTK_PACK_END);
+  gtk_notebook_set_action_widget (GTK_NOTEBOOK (terminal_window_get_notebook (window)), hbox, GTK_PACK_END);
 
   button = dropdown->keep_open = gtk_toggle_button_new ();
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
@@ -262,13 +262,13 @@ terminal_window_dropdown_init (TerminalWindowDropdown *dropdown)
   gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
 #endif
 
-  g_object_get (G_OBJECT (window->preferences), "dropdown-keep-open-default", &keep_open, NULL);
+  g_object_get (terminal_window_get_preferences (window), "dropdown-keep-open-default", &keep_open, NULL);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), keep_open);
 
   img = gtk_image_new_from_icon_name ("go-bottom", GTK_ICON_SIZE_MENU);
   gtk_container_add (GTK_CONTAINER (button), img);
 
-  action = gtk_action_group_get_action (window->action_group, "preferences");
+  action = terminal_window_get_action (window, "preferences");
 
   button = gtk_button_new ();
   gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
@@ -289,7 +289,7 @@ terminal_window_dropdown_init (TerminalWindowDropdown *dropdown)
   for (n = 1; n < N_PROPERTIES; n++)
     {
       name = g_param_spec_get_name (dropdown_props[n]);
-      g_object_bind_property (G_OBJECT (window->preferences), name,
+      g_object_bind_property (terminal_window_get_preferences (window), name,
                               G_OBJECT (dropdown), name,
                               G_BINDING_SYNC_CREATE);
     }
@@ -309,6 +309,7 @@ terminal_window_dropdown_set_property (GObject      *object,
   TerminalWindow         *window = TERMINAL_WINDOW (object);
   gdouble                 opacity;
   GdkScreen              *screen;
+  const gchar            *icon_name;
 
   switch (prop_id)
     {
@@ -337,9 +338,13 @@ terminal_window_dropdown_set_property (GObject      *object,
     case PROP_DROPDOWN_STATUS_ICON:
       if (g_value_get_boolean (value))
         {
-          if (dropdown->status_icon == NULL)
+          /* icon_name will be NULL initially after creation; will update it later */
+          icon_name = gtk_window_get_icon_name (GTK_WINDOW (object));
+          if (dropdown->status_icon == NULL && icon_name != NULL)
             {
-              dropdown->status_icon = gtk_status_icon_new_from_icon_name ("utilities-terminal");
+              dropdown->status_icon = g_path_is_absolute (icon_name)
+                ? gtk_status_icon_new_from_file (icon_name)
+                : gtk_status_icon_new_from_icon_name (icon_name);
               gtk_status_icon_set_title (dropdown->status_icon, _("Drop-down Terminal"));
               gtk_status_icon_set_tooltip_text (dropdown->status_icon, _("Toggle Drop-down Terminal"));
               g_signal_connect (G_OBJECT (dropdown->status_icon), "button-press-event",
@@ -357,8 +362,8 @@ terminal_window_dropdown_set_property (GObject      *object,
 
     case PROP_DROPDOWN_KEEP_ABOVE:
       gtk_window_set_keep_above (GTK_WINDOW (dropdown), g_value_get_boolean (value));
-      if (window->preferences_dialog != NULL)
-        terminal_util_activate_window (GTK_WINDOW (window->preferences_dialog));
+      if (terminal_window_get_preferences_dialog (window) != NULL)
+        terminal_util_activate_window (GTK_WINDOW (terminal_window_get_preferences_dialog (window)));
       return;
 
     case PROP_DROPDOWN_ANIMATION_TIME:
@@ -455,7 +460,7 @@ terminal_window_dropdown_focus_out_event (GtkWidget     *widget,
 
   /* check if keep open is not enabled */
   if (gtk_widget_get_visible (widget)
-      && TERMINAL_WINDOW (dropdown)->n_child_windows == 0
+      && !terminal_window_has_children (TERMINAL_WINDOW (dropdown))
       && gtk_grab_get_current () == NULL
       && dropdown->animation_dir != ANIMATION_DIR_UP) /* popup menu check */
     {
@@ -513,22 +518,21 @@ terminal_window_dropdown_status_icon_popup_menu (GtkStatusIcon          *status_
                                                  guint32                 timestamp,
                                                  TerminalWindowDropdown *dropdown)
 {
-  GtkActionGroup *group = TERMINAL_WINDOW (dropdown)->action_group;
-  GtkWidget      *menu;
-  GtkAction      *action;
+  GtkWidget *menu;
+  GtkAction *action;
 
   menu = gtk_menu_new ();
   g_signal_connect (G_OBJECT (menu), "selection-done",
       G_CALLBACK (gtk_widget_destroy), NULL);
 
-  action = gtk_action_group_get_action (group, "preferences");
+  action = terminal_window_get_action (TERMINAL_WINDOW (dropdown), "preferences");
   gtk_menu_shell_append (GTK_MENU_SHELL (menu),
                          gtk_action_create_menu_item (action));
 
   gtk_menu_shell_append (GTK_MENU_SHELL (menu),
                          gtk_separator_menu_item_new ());
 
-  action = gtk_action_group_get_action (group, "close-window");
+  action = terminal_window_get_action (TERMINAL_WINDOW (dropdown), "close-window");
   gtk_menu_shell_append (GTK_MENU_SHELL (menu),
                          gtk_action_create_menu_item (action));
 
@@ -550,6 +554,7 @@ terminal_window_dropdown_animate_down (gpointer data)
 {
   TerminalWindowDropdown *dropdown = TERMINAL_WINDOW_DROPDOWN (data);
   TerminalWindow         *window = TERMINAL_WINDOW (data);
+  GtkToggleAction        *action_fullscreen;
   GtkRequisition          req1;
   GdkRectangle            rect;
   gint                    step_size, vbox_h;
@@ -563,7 +568,9 @@ terminal_window_dropdown_animate_down (gpointer data)
 #else
   gdk_screen_get_monitor_geometry (dropdown->screen, dropdown->monitor_num, &rect);
 #endif
-  if (!gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (window->action_fullscreen)))
+
+  action_fullscreen = GTK_TOGGLE_ACTION (terminal_window_get_action (window, "fullscreen"));
+  if (!gtk_toggle_action_get_active (action_fullscreen))
     {
       /* calculate width/height if not fullscreen */
       rect.width *= dropdown->rel_width;
@@ -576,13 +583,13 @@ terminal_window_dropdown_animate_down (gpointer data)
     step_size = 1;
 
   /* new vbox size */
-  gtk_widget_get_preferred_size (window->vbox, &req1, NULL);
+  gtk_widget_get_preferred_size (terminal_window_get_vbox (window), &req1, NULL);
   vbox_h = req1.height + step_size;
   if (vbox_h > rect.height)
     vbox_h = rect.height;
 
   /* resize */
-  gtk_widget_set_size_request (window->vbox, req1.width, vbox_h);
+  gtk_widget_set_size_request (terminal_window_get_vbox (window), req1.width, vbox_h);
   gtk_window_resize (GTK_WINDOW (window), req1.width, vbox_h);
 
   /* continue the animation */
@@ -590,7 +597,7 @@ terminal_window_dropdown_animate_down (gpointer data)
     return TRUE;
 
   /* restore the fullscreen state */
-  if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (window->action_fullscreen)))
+  if (gtk_toggle_action_get_active (action_fullscreen))
     gtk_window_fullscreen (GTK_WINDOW (window));
 
   /* animation complete */
@@ -617,7 +624,7 @@ terminal_window_dropdown_animate_up (gpointer data)
 #else
   gdk_screen_get_monitor_geometry (dropdown->screen, dropdown->monitor_num, &rect);
 #endif
-  if (!gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (window->action_fullscreen)))
+  if (!gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (terminal_window_get_action (window, "fullscreen"))))
     {
       /* calculate width/height if not fullscreen */
       rect.width *= dropdown->rel_width;
@@ -630,24 +637,14 @@ terminal_window_dropdown_animate_up (gpointer data)
     step_size = 1;
 
   /* new vbox size */
-  gtk_widget_get_preferred_size (window->vbox, &req1, NULL);
+  gtk_widget_get_preferred_size (terminal_window_get_vbox (window), &req1, NULL);
   vbox_h = req1.height - step_size;
 
   /* sizes of the widgets that cannot be shrunk */
-  gtk_widget_get_preferred_size (window->notebook, &req1, NULL);
+  gtk_widget_get_preferred_size (terminal_window_get_notebook (window), &req1, NULL);
   min_size = req1.height;
-  if (window->menubar != NULL
-      && gtk_widget_get_visible (window->menubar))
-    {
-      gtk_widget_get_preferred_size (window->menubar, &req1, NULL);
-      min_size += req1.height;
-    }
-  if (window->toolbar != NULL
-      && gtk_widget_get_visible (window->toolbar))
-    {
-      gtk_widget_get_preferred_size (window->toolbar, &req1, NULL);
-      min_size += req1.height;
-    }
+  min_size += terminal_window_get_menubar_height (window);
+  min_size += terminal_window_get_toolbar_height (window);
 
   if (vbox_h < min_size)
     {
@@ -657,7 +654,7 @@ terminal_window_dropdown_animate_up (gpointer data)
     }
 
   /* resize window */
-  gtk_widget_set_size_request (window->vbox, rect.width, vbox_h);
+  gtk_widget_set_size_request (terminal_window_get_vbox (window), rect.width, vbox_h);
   gtk_window_resize (GTK_WINDOW (window), rect.width, vbox_h);
 
   return TRUE;
@@ -723,7 +720,7 @@ terminal_window_dropdown_show (TerminalWindowDropdown *dropdown,
       g_source_remove (dropdown->animation_timeout_id);
     }
 
-  g_object_get (G_OBJECT (window->preferences),
+  g_object_get (terminal_window_get_preferences (window),
                 "dropdown-move-to-active", &move_to_active,
                 NULL);
 
@@ -736,8 +733,7 @@ terminal_window_dropdown_show (TerminalWindowDropdown *dropdown,
   /* get the active monitor size */
 #if GTK_CHECK_VERSION (3, 22, 0)
   display = gdk_screen_get_display (dropdown->screen);
-  monitor = gdk_display_get_monitor_at_window (display,
-                                               gtk_widget_get_window (GTK_WIDGET (dropdown)));
+  monitor = gdk_display_get_monitor (display, dropdown->monitor_num);
   gdk_monitor_get_geometry (monitor, &monitor_geo);
 #else
   gdk_screen_get_monitor_geometry (dropdown->screen, dropdown->monitor_num, &monitor_geo);
@@ -747,7 +743,7 @@ terminal_window_dropdown_show (TerminalWindowDropdown *dropdown,
   gtk_window_set_screen (GTK_WINDOW (dropdown), dropdown->screen);
 
   /* correct padding with notebook size */
-  if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (window->action_fullscreen)))
+  if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (terminal_window_get_action (window, "fullscreen"))))
     {
       /* don't fullscreen during animation*/
       if (dropdown->animation_time > 0)
@@ -778,13 +774,13 @@ terminal_window_dropdown_show (TerminalWindowDropdown *dropdown,
       else if (old_animation_dir == ANIMATION_DIR_UP)
         {
           /* pick up where we aborted */
-          gtk_widget_get_preferred_size (window->vbox, &req1, NULL);
+          gtk_widget_get_preferred_size (terminal_window_get_vbox (window), &req1, NULL);
           vbox_h = req1.height;
         }
     }
 
   /* resize */
-  gtk_widget_set_size_request (window->vbox, w, vbox_h);
+  gtk_widget_set_size_request (terminal_window_get_vbox (window), w, vbox_h);
 
   /* calc position */
   x_dest = monitor_geo.x + (monitor_geo.width - w) * dropdown->rel_position;
@@ -811,7 +807,7 @@ terminal_window_dropdown_show (TerminalWindowDropdown *dropdown,
     }
   else
     {
-      g_object_get (G_OBJECT (window->preferences),
+      g_object_get (terminal_window_get_preferences (window),
                     "dropdown-keep-above", &keep_above,
                     NULL);
       gtk_window_set_keep_above (GTK_WINDOW (dropdown), keep_above);
@@ -835,7 +831,9 @@ terminal_window_dropdown_toggle_real (TerminalWindowDropdown *dropdown,
       && gdk_window_is_visible (gtk_widget_get_window (GTK_WIDGET (dropdown)))
       && dropdown->animation_dir != ANIMATION_DIR_UP)
     {
-      g_object_get (G_OBJECT (window->preferences), "dropdown-toggle-focus", &toggle_focus, NULL);
+      g_object_get (terminal_window_get_preferences (window),
+                    "dropdown-toggle-focus", &toggle_focus,
+                    NULL);
 
       /* if the focus was lost for 0.1 second and toggle-focus is used, we had
        * focus until the shortcut was pressed, and then we hide the window */
@@ -904,6 +902,7 @@ terminal_window_dropdown_get_timestamp (GtkWidget   *widget,
 
 GtkWidget *
 terminal_window_dropdown_new (const gchar        *role,
+                              const gchar        *icon,
                               gboolean            fullscreen,
                               TerminalVisibility  menubar,
                               TerminalVisibility  toolbar)
@@ -911,35 +910,49 @@ terminal_window_dropdown_new (const gchar        *role,
   TerminalWindow *window;
   gboolean        show_menubar;
   gboolean        show_toolbar;
+  gboolean        show_icon;
   GtkAction      *action;
+  GValue          value = G_VALUE_INIT;
 
   if (G_LIKELY (role == NULL))
     role = PACKAGE_NAME "-dropdown";
 
-  window = g_object_new (TERMINAL_TYPE_WINDOW_DROPDOWN, "role", role, NULL);
+  window = g_object_new (TERMINAL_TYPE_WINDOW_DROPDOWN,
+                         "role", role,
+                         "icon-name", icon != NULL ? icon : "utilities-terminal",
+                         NULL);
 
   /* read default preferences */
-  g_object_get (G_OBJECT (window->preferences),
+  g_object_get (terminal_window_get_preferences (window),
                 "misc-menubar-default", &show_menubar,
                 "misc-toolbar-default", &show_toolbar,
+                "dropdown-status-icon", &show_icon,
                 NULL);
 
+  /* update status bar icon */
+  g_value_init (&value, G_TYPE_BOOLEAN);
+  g_value_set_boolean (&value, show_icon);
+  terminal_window_dropdown_set_property (G_OBJECT (window),
+                                         PROP_DROPDOWN_STATUS_ICON, &value, NULL);
+
   /* setup full screen */
-  if (fullscreen && gtk_action_is_sensitive (window->action_fullscreen))
-    gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (window->action_fullscreen), TRUE);
+  action = terminal_window_get_action (window, "fullscreen");
+  if (fullscreen && gtk_action_is_sensitive (action))
+    gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), TRUE);
 
   /* setup menubar visibility */
   if (G_LIKELY (menubar != TERMINAL_VISIBILITY_DEFAULT))
     show_menubar = (menubar == TERMINAL_VISIBILITY_SHOW);
-  action = gtk_action_group_get_action (window->action_group, "show-menubar");
+  action = terminal_window_get_action (window, "show-menubar");
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), show_menubar);
   g_signal_connect_swapped (action, "activate",
       G_CALLBACK (terminal_window_dropdown_update_geometry), window);
+  terminal_window_action_show_menubar (GTK_TOGGLE_ACTION (action), window);
 
   /* setup toolbar visibility */
   if (G_LIKELY (toolbar != TERMINAL_VISIBILITY_DEFAULT))
     show_toolbar = (toolbar == TERMINAL_VISIBILITY_SHOW);
-  action = gtk_action_group_get_action (window->action_group, "show-toolbar");
+  action = terminal_window_get_action (window, "show-toolbar");
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), show_toolbar);
   g_signal_connect_swapped (action, "activate",
       G_CALLBACK (terminal_window_dropdown_update_geometry), window);
@@ -999,7 +1012,7 @@ terminal_window_dropdown_get_size (TerminalWindowDropdown *dropdown,
   terminal_screen_get_geometry (screen, &char_width, &char_height, &xpad, &ypad);
 
   /* correct padding with visible widgets */
-  gtk_widget_get_preferred_size (TERMINAL_WINDOW (dropdown)->vbox, &req, NULL);
+  gtk_widget_get_preferred_size (terminal_window_get_vbox (TERMINAL_WINDOW (dropdown)), &req, NULL);
   xpad += 2;
   ypad += req.height;
 
